package com.cool.store.mapper;

import com.cool.store.entity.UserRegionMappingDO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @author zhangchenbiao
 * @date 2023-06-07 02:07
 */
public interface UserRegionMappingMapper {
    /**
     *
     * 默认插入方法，只会给有值的字段赋值
	 * 会对传进来的字段做判空处理，如果字段为空，则使用数据库默认字段或者null
     * dateTime:2023-06-07 02:07
     */
    int batchInsertOrUpdateUserRegion(@Param("insertOrUpdateList") List<UserRegionMappingDO> insertOrUpdateList);

    /**
     *
     * 默认更新方法，根据主键更新，不会把null值更新到数据库，避免覆盖之前有值的
     * dateTime:2023-06-07 02:07
     */
    int updateByPrimaryKeySelective(UserRegionMappingDO record);

    /**
     * 删除用户区域
     * @param regionId
     * @param type
     * @param excludeUserIds
     * @return
     */
    int deleteUserRegion(@Param("regionId") String regionId, @Param("type")Integer type, @Param("excludeUserIds") List<String> excludeUserIds);

    /**
     * 删除用户不存在的部门
     * @param userId
     * @param type
     * @param excludeRegionIds
     * @return
     */
    int deleteUserRegionByExcludeRegionIds(@Param("userId") String userId, @Param("type")Integer type, @Param("excludeRegionIds") List<String> excludeRegionIds);

    /**
     * 删除没有的部门映射关系
     * @param excludeRegionIds
     * @return
     */
    Integer deleteRegionUserByExcludeRegionIds(@Param("excludeRegionIds") List<String> excludeRegionIds);

    /**
     * 删除用户不在的区域
     * @param excludeUserIds
     * @return
     */
    Integer deleteRegionUserByExcludeUserIds(@Param("excludeUserIds") List<String> excludeUserIds);

    /**
     * 删除用户所在的区域
     * @param userId
     * @return
     */
    Integer deleteUserRegionByUserId(@Param("userId") String userId);

    /**
     * 获取一个区域下是直挂的人
     * @param regionId
     * @return
     */
    List<String> getUserListByRegionId(@Param("regionId") String regionId);

    /**
     * 获取人所在的区域
     * @param userId
     * @return
     */
    List<String> getRegionIdsByUserId(@Param("userId") String userId);

    /**
     * 获取用户列表
     * @param regionIds
     * @return
     */
    List<String> getUserListByRegionIds(@Param("regionIds") List<String> regionIds);
}